import java.awt.Graphics2D;

public class Dashpot extends PhysicsElement implements Connector{
    private static int id=0;
    private final double viscosity;
    private Attachable a_end, b_end;
    private DashpotView view;
    
    private Dashpot(){
        this(0);
    }
    
    public Dashpot(double v){
        super(id++);
        viscosity=v;
        a_end=b_end=null;
        view = new DashpotView(this);
    }

    @Override
    public void paintView(Graphics2D g) {
        view.paint(g);
    }
    
    public Vector2D getAendPosition() {
        return a_end.getPosition();
    }
    
    public Vector2D getBendPosition() {
        return b_end.getPosition();
    }

    @Override
    public void attachEnd(Attachable sa) {
        if(a_end==null)
            a_end = sa; 
        else if(b_end==null)
            b_end=sa;
        sa.attach(this);
    }

    @Override
    public Vector2D getForce(Ball ball) {
        Vector2D force =new Vector2D(0, 0);
        if ((ball != a_end) && (ball != b_end))
            return force;
        if ((a_end == null) || (b_end == null))
            return force;
        return (ball==a_end)?a_end.getSpeed().times(-viscosity):b_end.getSpeed().times(-viscosity);
    }    
}
