import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintStream;
import java.util.ArrayList;
import javax.swing.Timer;


public class MyWorld implements ActionListener {
    private PrintStream out;  
    private ArrayList<PhysicsElement> elements;  // array to hold everything in my world.
    private Timer passingTime;   // NEW
    private double t;        // simulation time
    private double delta_t;        // in seconds
    private double refreshPeriod;  // in seconds
    private MyWorldView view;

    public MyWorld(){
        this(System.out);  
    }
    
    public MyWorld(PrintStream output){
        out = output;
        t = 0;
        refreshPeriod = 0.05;      // [s]
        delta_t = 0.005;          //  [s]
        elements = new ArrayList<>();
        view = null;
        passingTime = new Timer((int)(refreshPeriod*1000),this);
    }

    public void addElement(PhysicsElement e) {
        elements.add(e);
        view.repaint();
    }
    
    public void setView(MyWorldView view) {
        this.view = view;
    }
    
    public void start() {
        if(passingTime.isRunning()){
            passingTime.stop();
            return;
        }
        passingTime.start();      
    }
    
    @Override
    public void actionPerformed (ActionEvent event) {  // like simulate method of Assignment 1, 
        double nextStop=t+refreshPeriod;                // the arguments are attributes here.
        for (; t<nextStop; t+=delta_t){
            for (PhysicsElement e: elements)
                if (e instanceof Simulateable) {
                    Simulateable s = (Simulateable) e;
                    s.computeNextState(delta_t); // compute each element next state based on current global state
                }
            for (PhysicsElement e: elements){  // for each element update its state. 
                if (e instanceof Simulateable) {
                    Simulateable s = (Simulateable) e;
                    s.updateState();            // update its state
                }
            }
        }
        view.repaint();
    }

    public ArrayList<PhysicsElement> getPhysicsElements(){
        return elements;
    }
} 

