import java.awt.Graphics2D;
import java.util.ArrayList;


public class Oscillator extends PhysicsElement implements Attachable,Simulateable{
    private static int id=0;
    private Vector2D center;
    private Vector2D amplitude;
    private double w;
    private double time;
    private Vector2D pos_t;
    private Vector2D pos_tPlusDelta;
    private ArrayList<Connector> con=null;
    private OscillatorView view;
    
    public Oscillator(){
        this(new Vector2D(0.0,0.0),new Vector2D(1.0,1.0),0.5);
    }
    
    public Oscillator(Vector2D c,Vector2D a, double f){
        super(id++);
        amplitude=a;
        pos_t=pos_tPlusDelta=center=c;
        w=2*Math.PI*f;
        time=0;
        view=new OscillatorView(this, 0.08);
        con = new ArrayList<>();
    }

    @Override
    public void paintView(Graphics2D g) {
        view.paint(g);
    }

    @Override
    public void attach(Connector c) {
        con.add(c);
    }

    @Override
    public Vector2D getPosition() {
        return this.pos_t;
    }

    @Override
    public Vector2D getSpeed() {
        return new Vector2D(0.0, 0.0);
    }

    @Override
    public void computeNextState(double delta_t) {
        time+=delta_t;
        pos_tPlusDelta=center.plus(amplitude.times(Math.sin(w*time)));
    }

    @Override
    public void updateState() {
        pos_t=pos_tPlusDelta;
    }
    
}
