import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 * @author Edgard Abarcas, Nicolas Olivos
 * @version 1.4 Cuarto paso de la tarea 29 de Abril de 2015
 */

public class Ball extends PhysicsElement implements Simulateable,Attachable {
    //Inicio documentacion de atributos.
    /**
     * Identificador del elemento fisico.
     */
    private static int id=0;
    /**
     * Masa de la bola.
     */
    private final double mass;
    /**
     * Vector direccional de la posicion con respecto al tiempo presente.
     */
    private Vector2D pos_t;
    /**
     * Vector direccional de la posicion con respecto al tiempo futuro.
     */
    private Vector2D pos_tPlusDelta;
    /**
     * Vector direccional de la velocidad con respecto al tiempo presente.
     */
    private Vector2D speed_t;
    /**
     * Vector direccional de la velocidad con respecto al tiempo futuro.
     */
    private Vector2D speed_tPlusDelta;
    /**
     * Vector direccional de la aceleracion con respecto al tiempo presente.
     * @since version 1.2 Segundo paso de la tarea.
     */
    private Vector2D acc_t;
    /**
     * ArrayList que almacena todos los conectores de la bola.
     * @since version 1.2 Segundo paso de la tarea.
     */
    private ArrayList<Connector> con=null;
    /**
     * Vista grafica de la bola.
     */
    private BallView view;
    //Fin documentacion de atributos.
    
    //Inicio documentacion de constructores.
    /**
     * Constructor privado por defecto.
     */
    private Ball(){
        this(1.0,new Vector2D(),new Vector2D());
    }
    
    /**
     * Constructor publico de la clase.
     * @param mass masa de la bola.
     * @param position posicion inicial de la bola.
     * @param speed velocidad inicial de la bola.
     */
    public Ball(double mass, Vector2D position, Vector2D speed){
        super(id++);
        this.mass = mass;
        pos_t = position;
        speed_t = speed;
        view=new BallView(this,0.05);
        con = new ArrayList<>();
    }
    //Fin documentacion de constructores.
    
    //Inicio documentacion de metodos de la clase.
    /**
     * Metodo que cambia el color visual de la bola.
     * Por ejemplo:
     * <PRE> Vector2D position =new Vector2D( 0.0,0.0);
     * Vector2D speed = new Vector2D( 0.5,0.5);
     * Double mass =1.0;
     * Ball b = new Ball(mass, position, speed);
     * b.setColor(Color.RED);</PRE>
     * @param color nuevo color de la bola.
     * @since version 1.1 Primer paso de la tarea.
     */
    public void setColor(Color color){
        this.view.setColor(color);
    }
    
    /**
     * Metodo que calcula la fuerza neta ejercida sobre la bola.
     * @return Vector2D netForce, vector direccional de la fuerza neta aplicada en la bola.
     * @since version 1.2 Segundo paso de la tarea.
     */
    private Vector2D getNetForce(){
        Vector2D netForce=new Vector2D(0, 0);
        for (Connector e: con)
            netForce=netForce.plus(e.getForce(this));
        return netForce;
    }
    //Fin documentacion de metodos de la clase.
    
    //Inicio documentacion de metodos heredados de PhysicsElement.
    /**
     * Metodo sobreescrito heredado de PhysicsElement.
     * Permite la incorporacion visual de la bola al grafico del contenedor de la bola.
     * @param g grafico del contenedor de la bola.
     */
    @Override
    public void paintView(Graphics2D g){
        view.paint(g);
    }
    //Fin documentacion de metodos heredados de PhysicsElement.
    
    //Inicio documentacion de metodos implementados de la interfaz Simulateable.
    /**
     * Metodo sobreescrito implementado de la interfaz Simulateable.
     * Permite el calculo de los atributos relacionados con el tiempo futuro de la bola.
     * @param delta_t tiempo futuro.
     */
    @Override
    public void computeNextState(double delta_t) {
        if (con.isEmpty()){
            pos_tPlusDelta= pos_t.plus(speed_t.times(delta_t));
            speed_tPlusDelta=speed_t;
        }
        else{
            acc_t = getNetForce().times(1/mass);
            pos_tPlusDelta= pos_t.plus(speed_t.times(delta_t)).plus(acc_t.times(delta_t*delta_t));
            speed_tPlusDelta=speed_t.plus(acc_t.times(delta_t));
        }
    }
    
    /**
     * Metodo sobreescrito implementado de la interfaz Simulateable.
     * Actualiza los atributos relacionados con el tiempo presente de la bola.
     */
    @Override
    public void updateState(){
        pos_t=pos_tPlusDelta;
        speed_t=speed_tPlusDelta;
    }
    //Fin documentacion de metodos implementados de la interfaz Simulateable.

    //Inicio documentacion de metodos implementados de la interfaz Attachable.
    /**
     * Metodo sobreescrito implementado de la interfaz Attachable.
     * Permite agregar conectores relacionados con la bola.
     * @param c ,conector a agregar.
     */
    @Override
    public void attach(Connector c) {
        con.add(c);
    }
    
    /**
     * Metodo sobreescrito implementado de la interfaz Attachable.
     * Permite obtener la posicion de la pelota.
     * @return Vector2D pos_t, vector direccional de la posicion de la bola.
     */
    @Override
    public Vector2D getPosition() {
        return pos_t;
    }

    /**
     * Metodo sobreescrito implementado de la interfaz Attachable.
     * Permite obtener la velocidad de la pelota.
     * @return Vector2D speed_t, vector direccional de la velocidad de la bola.
     */
    @Override
    public Vector2D getSpeed() {
        return this.speed_t;
    }
    //Fin documentacion de metodos implementados de la interfaz Attachable.
}
