import java.awt.Graphics2D;

/**
*@author Nicolas Olivos
*/
public class Elastic extends PhysicsElement implements Connector{
  private static int id=0;
  protected final double restLength;
  private final double stiffness;
  protected Attachable a_end, b_end;
  protected ElasticView view;
  
  private Elastic(){
      this(0,0);
  }
  
  public Elastic(double len,double k){
      super(id++);
      restLength=len;
      stiffness=k;
      a_end = b_end = null;
      view = new ElasticView(this);
  }
  
  public double getRestLength(){
    return this.restLength;
  }
  
  public Vector2D getAendPosition(){
    return this.a_end.getPosition();
  }
  
  public Vector2D getBendPosition(){
    return this.b_end.getPosition();
  }
  
  private Vector2D getStrech() {
    Vector2D va=null,vb=null,amenosb,s;
    double factor,normaAmenosB;
    if (a_end==null||b_end==null)
      return new Vector2D(0.0,0.0);
    if (a_end!=null)
      va=a_end.getPosition();
    if (b_end!=null)
      vb=b_end.getPosition();
    if(va==null)
      return new Vector2D(0.0,0.0);
    amenosb=va.minus(vb);
    normaAmenosB=amenosb.module();
    factor=(normaAmenosB-this.restLength)/normaAmenosB;
    s=amenosb.times(factor);
    return s;
  }
  
  private double getStiffness(){
    Vector2D va=null,vb=null,amenosb,s;
    double factor,normaAmenosB;
    if (a_end==null||b_end==null)
      return 0;
    if (a_end!=null)
      va=a_end.getPosition();
    if (b_end!=null)
      vb=b_end.getPosition();
    if(va==null)
      return 0;
    normaAmenosB=va.minus(vb).module();
    return (normaAmenosB>restLength)?stiffness:0;
  } 
  
    @Override
  public void attachEnd (Attachable sa){
    if(a_end==null)
        a_end = sa; 
    else if(b_end==null)
        b_end=sa;
    sa.attach(this);
  }
  
    @Override
  public Vector2D getForce(Ball ball) {
    Vector2D force=getStrech().times(getStiffness());
    return (ball==a_end)?force.times(-1):force;
  }
  
    @Override
  public void paintView (Graphics2D g) {
    view.paint(g);     
  }
}